import htmlgenerator as hg
from bread import layout as layout
from bread.utils.urls import aslayout
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.forms import global_preference_form_builder

import basxconnect.core.layouts.settings_layout as settings_layout

R = layout.grid.Row
C = layout.grid.Col
F = layout.form.FormField


@aslayout
def generalsettings(request):

    if request.method == "POST":
        form = global_preference_form_builder(
            preferences=["general__organizationname"]
        )(request.POST, request.FILES)
        if form.is_valid():
            form.update_preferences()
    else:
        form = global_preference_form_builder(
            preferences=["general__organizationname"]
        )()

    return hg.WithContext(
        layout.grid.Grid(
            R(C(hg.H3(_("Settings")))),
            R(C(hg.H4(_("General")))),
            R(C(hg.H5(_("Information about our organization")))),
            R(
                C(
                    layout.form.Form.wrap_with_form(
                        form,
                        hg.BaseElement(
                            layout.form.FormField("general__organizationname")
                        ),
                        style="max-width: 480px",
                    )
                )
            ),
            gutter=False,
        ),
        form=form,
    )


@aslayout
def personsettings(request):
    return settings_layout.personsettings(request)


@aslayout
def relationshipssettings(request):
    return settings_layout.relationshipssettings(request)



from typing import List, NamedTuple

import django_countries

import basxconnect.core.layouts.editperson.common.base_data
from basxconnect.core import models
from basxconnect.mailer_integration.abstract.abstract_datasource import (
    Datasource,
    MailerPerson,
)
from basxconnect.mailer_integration.models import Interest, MailingPreferences


class SynchronizationResult(NamedTuple):
    new_persons: int
    total_synchronized_persons: int
    invalid_new_persons: List[str]


def download_persons(datasource: Datasource) -> SynchronizationResult:
    MailingPreferences.objects.all().delete()
    Interest.objects.all().delete()
    interests = datasource.get_interests()
    for interest in interests:
        Interest.objects.get_or_create(external_id=interest.id, name=interest.name)

    mailer_persons = datasource.get_persons()
    datasource_tag = _get_or_create_tag(datasource.tag())
    new_persons = 0
    invalid_new_persons = []
    for mailer_person in mailer_persons:
        matching_email_addresses = list(
            models.Email.objects.filter(
                email=mailer_person.email,
            ).all()
        )
        if len(matching_email_addresses) == 0:
            if not is_valid_new_person(mailer_person):
                invalid_new_persons.append(mailer_person.email)
            else:
                _save_person(datasource_tag, mailer_person)
                new_persons += 1
        else:
            # if the downloaded email address already exists in our system, update the mailing preference for this email
            # address, without creating a new person in the database
            for email in matching_email_addresses:
                _save_mailing_preferences(email, mailer_person)

    return SynchronizationResult(new_persons, len(mailer_persons), invalid_new_persons)


def is_valid_new_person(person: MailerPerson):
    return django_countries.Countries().countries.get(
        person.country
    ) and person.status in [
        "subscribed",
        "unsubscribed",
    ]


def _get_or_create_tag(tag: str) -> models.Term:
    tags_vocabulary = models.Vocabulary.objects.get(slug="tag")
    tag, _ = models.Term.objects.get_or_create(term=tag, vocabulary=tags_vocabulary)
    return tag


def _save_person(datasource_tag: models.Term, mailer_person: MailerPerson):
    person = models.NaturalPerson.objects.create(
        first_name=mailer_person.first_name,
        name=mailer_person.display_name,
        last_name=mailer_person.last_name,
    )
    basxconnect.core.layouts.editperson.common.base_data.tags.add(datasource_tag)
    person.save()
    email = models.Email.objects.create(email=mailer_person.email, person=person)
    person.primary_email_address = email
    person.save()
    _save_mailing_preferences(email, mailer_person)
    _save_postal_address(person, mailer_person)


def _save_postal_address(person: models.Person, mailer_person: MailerPerson):
    address = models.Postal(
        person=person,
        country=mailer_person.country,
        address=mailer_person.address,
        postcode=mailer_person.postcode,
        city=mailer_person.city,
    )
    address.save()
    person.primary_postal_address = address
    person.save()


def _save_mailing_preferences(email: models.Email, mailer_person: MailerPerson):
    mailing_preferences, _ = MailingPreferences.objects.get_or_create(email=email)
    mailing_preferences.status = mailer_person.status
    mailing_preferences.interests.clear()
    for interest_id in mailer_person.interests_ids:
        interest = Interest.objects.get(external_id=interest_id)
        mailing_preferences.interests.add(interest)
    mailing_preferences.save()



from bread import layout as layout
from bread import menu
from bread.utils.links import Link
from bread.utils.urls import reverse, reverse_model
from django.utils.translation import gettext_lazy as _

from .. import models

R = layout.grid.Row
C = layout.grid.Col
F = layout.form.FormField

# MENU ENTRIES ---------------------------------------------------------------------

persongroup = menu.Group(_("Persons"), iconname="group")
settingsgroup = menu.Group(_("Settings"), iconname="settings", order=100)

menu.registeritem(
    menu.Item(
        Link(reverse_model(models.Person, "browse"), _("Persons"), iconname="group"),
        persongroup,
    )
)

menu.registeritem(
    menu.Item(
        Link(
            reverse("basxconnect.core.views.settings_views.generalsettings"),
            _("General"),
        ),
        settingsgroup,
    )
)
menu.registeritem(
    menu.Item(
        Link(
            reverse("basxconnect.core.views.settings_views.personsettings"),
            _("Persons"),
        ),
        settingsgroup,
    )
)
menu.registeritem(
    menu.Item(
        Link(
            reverse("basxconnect.core.views.settings_views.relationshipssettings"),
            _("Relationships"),
        ),
        settingsgroup,
    )
)



from django import forms
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.preferences import Section
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import (
    FilePreference,
    LongStringPreference,
    StringPreference,
)

general = Section("general", _("General"))


@global_preferences_registry.register
class SenderLetterhead(LongStringPreference):
    section = general
    name = "sender_letterhead"
    default = ""
    verbose_name = _("Sender letterhead")


@global_preferences_registry.register
class Logo(FilePreference):
    section = general
    name = "logo"
    field_class = forms.ImageField
    default = ""
    verbose_name = _("Logo")

    field_kwargs = {
        "required": False,
    }


@global_preferences_registry.register
class OrganizationName(StringPreference):
    section = general
    name = "organizationname"
    default = "<Your Organization>"
    verbose_name = _("Organization Name")



import traceback

import bread.layout.components.notification
import htmlgenerator as hg
from bread import layout, menu
from bread.layout.components.forms import Form
from bread.utils import aslayout, reverse_model
from bread.utils.links import Link
from bread.views import AddView, EditView
from django import forms
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from basxconnect.core.views import menu_views
from basxconnect.mailer_integration import download_data, settings
from basxconnect.mailer_integration.abstract.abstract_datasource import MailerPerson
from basxconnect.mailer_integration.mailchimp import datasource


@aslayout
def mailchimp_view(request):
    if request.method == "POST":
        try:
            sync_result = download_data.download_persons(settings.MAILER)
            notification = bread.layout.components.notification.InlineNotification(
                "Success",
                f"Synchronized mailing preferences for {sync_result.total_synchronized_persons} Mailchimp "
                f"contacts. {sync_result.new_persons} new persons were added to the BasxConnect database. "
                + (
                    "The following mailchimp contacts are not yet in our database but were also not "
                    "added because they were invalid:"
                    + (", ".join(sync_result.invalid_new_persons))
                    if len(sync_result.invalid_new_persons) > 0
                    else ""
                ),
            )
        except Exception:
            notification = bread.layout.components.notification.InlineNotification(
                "Error",
                f"An error occured during synchronization. {traceback.format_exc()}",
                kind="error",
            )
    else:
        notification = None

    return Form.wrap_with_form(
        forms.Form(),
        hg.If(notification is not None, notification),
        submit_label="Synchronize with Mailchimp",
    )


menu.registeritem(
    menu.Item(
        Link(
            reverse_lazy("basxconnect.mailer_integration.views.mailchimp_view"),
            _("Mailchimp"),
        ),
        menu_views.settingsgroup,
    )
)


class AddMailingPreferencesView(AddView):
    def get_success_url(self):
        return reverse_model(
            self.object.email.person, "read", kwargs={"pk": self.object.email.person.pk}
        )

    def post(self, request, *args, **kwargs):
        response = super().post(request, *args, **kwargs)
        settings.MAILER.add_person(MailerPerson.from_mailing_preferences(self.object))
        return response


class EditMailingPreferencesView(EditView):
    fields = ["interests"]

    def get_success_url(self):
        return reverse_model(
            self.object.email.person, "read", kwargs={"pk": self.object.email.person.pk}
        )

    def post(self, request, *args, **kwargs):
        result = super().post(request, *args, **kwargs)
        # TODO: https://github.com/basxsoftwareassociation/basxconnect/issues/140
        datasource.MailchimpDatasource().put_person(
            MailerPerson.from_mailing_preferences(self.object)
        )
        return result

    def get_layout(self):
        form_fields = [layout.form.FormField(field) for field in self.fields]
        return hg.DIV(*form_fields)



from django.db import models
from django.utils.translation import gettext_lazy as _

from basxconnect.core.models import Email


class Interest(models.Model):
    name = models.CharField(max_length=100, unique=True)
    external_id = models.CharField(max_length=50)

    def __str__(self):
        return self.name


class MailingPreferences(models.Model):
    email = models.OneToOneField(Email, on_delete=models.CASCADE)
    status = models.CharField(
        max_length=50,
        choices=[
            ("subscribed", "subscribed"),
            ("unsubscribed", "unsubscribed"),
            ("non-subscribed", "non-subscribed"),
            ("cleaned", "cleaned"),
        ],
    )

    interests = models.ManyToManyField(Interest, blank=True)
    interests.verbose_name = _("Mailing Interests")



from bread.utils.urls import autopath

import basxconnect.mailer_integration.views
from basxconnect.mailer_integration import models

urlpatterns = [
    autopath(basxconnect.mailer_integration.views.mailchimp_view),
    autopath(
        basxconnect.mailer_integration.views.EditMailingPreferencesView.as_view(
            model=models.MailingPreferences
        )
    ),
    autopath(
        basxconnect.mailer_integration.views.AddMailingPreferencesView.as_view(
            model=models.MailingPreferences
        )
    ),
]



from bread import layout
from bread.utils import Link, ModelHref, pretty_modelname
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _

from basxconnect.contributions.models import Contribution
from basxconnect.core.models import Person

R = layout.grid.Row
C = layout.grid.Col
F = layout.form.FormField


def contributions_tab(request):
    person = get_object_or_404(Person, pk=request.resolver_match.kwargs["pk"])
    return layout.tabs.Tab(
        _("Contributions"),
        layout.datatable.DataTable.from_queryset(
            person.contributions.all(),
            columns=[
                "_import.date",
                "date",
                "note",
                "debitaccount",
                "creditaccount",
                "amount_formatted",
            ],
            title="",
            primary_button=layout.button.Button.fromlink(
                Link(
                    href=ModelHref(Contribution, "add", query={"person": person.id}),
                    label=_("Add %s") % pretty_modelname(Contribution),
                ),
                icon=layout.icon.Icon("add", size=20),
            ),
            backurl=request.get_full_path(),
            prevent_automatic_sortingnames=True,
        ),
    )



from celery import shared_task
from django.shortcuts import get_object_or_404


@shared_task
def save_person(person_id):
    from basxconnect.core import models

    person = get_object_or_404(models.Person, pk=person_id)
    person.save()



import django.forms
import htmlgenerator as hg
from bread import layout
from bread.layout.components.icon import Icon
from bread.views import AddView, EditView
from django.utils.translation import gettext_lazy as _

from basxconnect.core import models


class NaturalPersonEditMailingsView(EditView):
    model = models.NaturalPerson
    fields = [
        "preferred_language",
        "type",
        "salutation_letter",
        "gender",
        "form_of_address",
    ]

    @staticmethod
    def path():
        return "person_modals_views.naturalpersoneditmailingsview"

    @staticmethod
    def read_heading():
        return _("Settings")

    @staticmethod
    def edit_heading():
        return _("Edit mailing settings")

    @staticmethod
    def icon():
        return Icon("settings--adjust")


class NaturalPersonEditPersonalDataView(EditView):
    model = models.NaturalPerson
    fields = [
        "salutation",
        "title",
        "name",
        "autogenerate_displayname",
        "first_name",
        "last_name",
        "date_of_birth",
        "profession",
        "deceased",
        "decease_date",
    ]

    def get_form(self, *args, **kwargs):
        form = super().get_form(*args, **kwargs)
        # only set the disabled attribute on the widget, otherwise we cannot change it on the client side in the future
        if self.object.autogenerate_displayname:
            form.fields["name"].widget.attrs["disabled"] = True
        form.fields["autogenerate_displayname"].widget.attrs[
            "onchange"
        ] = "if(this.checked) { $('input[name=name]').setAttribute('disabled', '') } else { $('input[name=name]').removeAttribute('disabled') }"
        return form

    def get_layout(self):
        R = layout.grid.Row
        C = layout.grid.Col
        F = layout.form.FormField
        return layout.grid.Grid(
            R(C(F("salutation"))),
            R(C(F("title"))),
            R(
                C(
                    F("name"),
                    width=10,
                    breakpoint="lg",
                ),
                C(
                    F("autogenerate_displayname"),
                    width=6,
                    breakpoint="lg",
                    style="align-self: flex-end;",
                ),
            ),
            R(C(F("first_name"))),
            R(C(F("last_name"))),
            R(C(F("date_of_birth"))),
            R(C(F("profession"))),
            R(C(F("deceased"))),
            R(C(F("decease_date"))),
        )


class LegalPersonEditPersonalDataView(EditView):
    model = models.LegalPerson
    fields = [
        "name",
        "name_addition",
    ]


class PersonAssociationEditPersonalDataView(EditView):
    model = models.PersonAssociation
    fields = [
        "name",
        "preferred_language",
        "salutation_letter",
    ]


class LegalPersonEditMailingsView(EditView):
    model = models.LegalPerson
    fields = [
        "preferred_language",
        "type",
        "salutation_letter",
    ]


class EditPostalAddressView(EditView):
    model = models.Postal
    fields = [
        "type",
        "address",
        "postcode",
        "city",
        "country",
        "valid_from",
        "valid_until",
    ]

    def form_valid(self, form, *args, **kwargs):
        ret = super().form_valid(form, *args, **kwargs)
        is_primary = form.cleaned_data["is_primary"]
        if is_primary:
            self.object.person.primary_postal_address = self.object
        self.object.person.save()
        return ret

    def get_form_class(self, *args, **kwargs):
        class EditPostalForm(super().get_form_class(*args, **kwargs)):
            is_primary = django.forms.BooleanField(
                label=_("Use as primary postal address"), required=False
            )

        return EditPostalForm

    def get_layout(self):
        form_fields = [layout.form.FormField(field) for field in [*self.fields]] + [
            hg.If(
                hg.F(
                    lambda c: c["object"].person.primary_postal_address
                    and c["object"].person.primary_postal_address.pk != c["object"].pk
                ),
                layout.form.FormField("is_primary"),
                "",
            )
        ]
        return hg.DIV(*form_fields)


class AddPostalAddressView(AddView):
    model = models.Postal

    def post(self, request, *args, **kwargs):
        ret = super().post(request, *args, **kwargs)
        self.object.person.save()
        return ret


class EditEmailAddressView(EditView):
    model = models.Email
    fields = ["type", "email"]

    def form_valid(self, form, *args, **kwargs):
        ret = super().form_valid(form, *args, **kwargs)
        is_primary = form.cleaned_data["is_primary"]
        if is_primary:
            self.object.person.primary_email_address = self.object
        self.object.person.save()
        return ret

    def get_form_class(self, *args, **kwargs):
        class EditEmailForm(super().get_form_class(*args, **kwargs)):
            is_primary = django.forms.BooleanField(
                label=_("Use as primary email address"), required=False
            )

        return EditEmailForm

    def get_layout(self):
        form_fields = [layout.form.FormField(field) for field in [*self.fields]] + [
            hg.If(
                hg.F(
                    lambda c: c["object"].person.primary_email_address
                    and c["object"].person.primary_email_address.pk != c["object"].pk
                ),
                layout.form.FormField("is_primary"),
                "",
            )
        ]
        return hg.BaseElement(
            hg.H3(_("Edit Email")),
            layout.form.Form.wrap_with_form(hg.C("form"), hg.DIV(*form_fields)),
        )


class AddEmailAddressView(AddView):
    model = models.Email

    def post(self, request, *args, **kwargs):
        ret = super().post(request, *args, **kwargs)
        self.object.person.save()
        return ret


class NaturalPersonEditRemarksView(EditView):
    model = models.NaturalPerson
    fields = [
        "remarks",
    ]


class LegalPersonEditRemarksView(EditView):
    model = models.LegalPerson
    fields = [
        "remarks",
    ]


class PersonAssociationEditRemarksView(EditView):
    model = models.PersonAssociation
    fields = [
        "remarks",
    ]


class NaturalPersonEditTagsView(EditView):
    model = models.NaturalPerson
    fields = [
        "tags",
    ]


class LegalPersonEditTagsView(EditView):
    model = models.LegalPerson
    fields = [
        "tags",
    ]


class PersonAssociationEditTagsView(EditView):
    model = models.PersonAssociation
    fields = [
        "tags",
    ]



import bread
import htmlgenerator as hg
from bread import layout
from bread.layout.components import tag
from bread.layout.components.icon import Icon
from bread.utils import reverse
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _

from basxconnect.core import models
from basxconnect.core.layouts.editperson.common.utils import tile_with_icon
from basxconnect.core.models import Person
from basxconnect.mailer_integration.models import Interest

R = bread.layout.grid.Row
C = bread.layout.grid.Col


def mailer_integration_tile(request):
    person = get_object_or_404(Person, pk=request.resolver_match.kwargs["pk"])
    addresses = person.core_email_list.all()
    return tile_with_icon(
        Icon("email--new"),
        hg.H4(
            _("Email Subscriptions"),
        ),
        *[_display_preferences(email) for email in addresses]
        if hasattr(person, "core_email_list") and person.core_email_list.count() > 0
        else C(_("Person has no email addresses")),
    )


def _display_preferences(email):
    if not hasattr(email, "mailingpreferences"):
        return _display_email_without_preferences(email)
    mailingpreferences = email.mailingpreferences
    modal = modal_edit_mailingpreferences(mailingpreferences)
    interests = [
        R(
            C(interest, width=6, breakpoint="lg"),
            C(
                is_interested_indicator(interest in mailingpreferences.interests.all()),
                breakpoint="lg",
            ),
            style="padding-bottom: 24px;",
        )
        for interest in Interest.objects.all()
    ]
    return hg.BaseElement(
        R(
            C(
                hg.SPAN(email.email, style="font-weight: bold;"),
                tag.Tag(
                    _(mailingpreferences.status),
                    tag_color=map_tag_color(mailingpreferences.status),
                    onclick="return false;",
                    style="margin-left: 1rem;",
                ),
                style="margin-bottom: 1.5rem;",
            ),
        ),
        *interests,
        R(
            C(
                layout.button.Button(
                    "Edit",
                    buttontype="tertiary",
                    icon="edit",
                    **modal.openerattributes,
                ),
                modal,
                style="margin-top: 1.5rem;margin-bottom: 3rem;",
            )
        ),
    )


def _display_email_without_preferences(email):
    modal_add = modal_add_mailingpreferences(email)
    return hg.BaseElement(
        hg.DIV(email.email, style="font-weight: bold;"),
        hg.DIV(_("no mailing preferences yet for "), email.email),
        layout.button.Button(
            _("Add mailing preferences"),
            buttontype="ghost",
            icon="add",
            **modal_add.openerattributes,
        ),
        modal_add,
    )


def map_tag_color(status):
    mapping = {"subscribed": "green"}
    return mapping.get(status, "gray")


def is_interested_indicator(is_subscribed):
    if is_subscribed:
        color = "#198038"
        text = _("active")
    else:
        color = "#e0e0e0"
        text = _("inactive")
    return hg.DIV(
        hg.DIV(
            "",
            style=f"height: 8px; width: 8px; background-color: {color}; border-radius: 50%; display: inline-block;",
        ),
        hg.DIV(text, style="display: inline-block; padding-left: 8px"),
        style="display: inline-block;",
    )


def modal_edit_mailingpreferences(mailingpreferences):
    modal = layout.modal.Modal.with_ajax_content(
        heading=_("Edit Email Subscriptions"),
        url=reverse(
            "basxconnect.mailer_integration.views.editmailingpreferencesview",
            kwargs={"pk": mailingpreferences.pk},
            query={"asajax": True},
        ),
        submitlabel=_("Save"),
    )
    modal[0][1].attributes["style"] = "overflow: visible"
    modal[0].attributes["style"] = "overflow: visible"
    return modal


def modal_add_mailingpreferences(email: models.Email):
    ret = layout.modal.Modal.with_ajax_content(
        heading=_("Add Mailing Preferences"),
        url=reverse(
            "basxconnect.mailer_integration.views.addmailingpreferencesview",
            query={"asajax": True, "email": email.pk, "status": "subscribed"},
        ),
        submitlabel=_("Save"),
    )
    ret[0][1].attributes["style"] = "overflow: visible"
    ret[0].attributes["style"] = "overflow: visible"
    return ret
